/* ---------------------------------------------------------
  Compute the depth of a surface crevasse using the
  Weertman (1973) approach. Written by Stephen Jimenez.
------------------------------------------------------------ */


/* Include libraries. */
#include <iostream>
#include <math.h>


/* Define constants. */
const double rho_ice = 917; // ice density (kg/m^3)
const double rho_H2O = 1020; // seawater density  (kg/m^3)
const double g = 9.81; // gravity acceleration (m/s^2)
const double pi = M_PI; // pi
const double E = 9500; // Young's modulus for ice (GPa)
const double nu = 0.35; // Poisson's ratio for ice
const double mu = E/2/(1+nu); // shear modulus for ice (GPa)


/* Forward declarations. */
double Bisection(double H, double hw, double hs);
double Dislocation(double y, double ds, double hs, double Rxx);
double MinD(double ds, double hs, double Rxx);
double T_(double ds, double hs);


/* ---------------------------------------------------------
  Define subroutines.
------------------------------------------------------------ */


// bisection method to compute crevasse depth
double Bisection(double H, double hw, double hs) {
  double Rxx = (0.5*rho_ice*g*H - 0.5*rho_H2O*g*hw*hw/H); // restrictive stress
  double da = 1e-6, db = 0.5*H, dc = H*(1-1e-6); // initial bounds for bisection
  double Da, Db, Dc;
  double error = 1, tol = 1e-6;
  int k = 0, maxk = 50; // iterations
  while (fabs(error) > tol && k < maxk) {
    // compute minimum dislocation for crevasse depths da, db, and dc
    Da = MinD(da, hs, Rxx);
    Db = MinD(db, hs, Rxx);
    Dc = MinD(dc, hs, Rxx);
    // full penetration case
    if (Da > 0 && Db > 0 && Dc > 0) return H;
    // no penetration case
    else if (Da < 0 && Db < 0 && Dc < 0) return 0;
    // crevasse depth ds is between db and dc
    else if (Da > 0 && Db > 0 && Dc < 0) {
      da = db;
      db = 0.5*(da + dc);
      error = fabs((dc - da)/db);
    }
    // crevasse depth ds is between da and db
    else if (Da > 0 && Db < 0 && Dc < 0) {
      dc = db;
      db = 0.5*(da + dc);
      error = fabs((dc - da)/db);
    }
    k++; // update iteration count
  }
  return db;
}


// dislocation of crevasse walls
double Dislocation(double y, double ds, double hs, double Rxx) {
  double h_ = ds*(1-hs); // distance between crevasse water level and top of slab
  double term1 = sqrt(ds*ds - y*y);
  double term2 = sqrt(ds*ds - h_*h_);
  double term3 = term1*(ds - rho_H2O/rho_ice*term2);
  double term4 = y*y*log((ds + term1)/y);
  double term5 = 0.5*rho_H2O/rho_ice*(y*y
    + h_*h_)*log(fabs((term2 + term1)/(term2 - term1)));
  double term6 = y*h_*rho_H2O/rho_ice*log(fabs((y*term2
    + h_*term1)/(y*term2 - h_*term1)));
  double term7 = (Rxx - T_(ds, hs))*term1;
  return 2*(1-nu)/mu*(rho_ice*g/pi*(term3 - term4 + term5 - term6) + term7);
}


// get the maximum dislocation
double MinD(double ds, double hs, double Rxx) {
  int m = 1e3; // number of sample points between 0 and ds
  double dy = ds/m, miny = 1;
  double y = dy;
  for (int i=0; i<m-1; i++) {
    miny = std::min(miny, Dislocation(y, ds, hs, Rxx));
    y += dy;
  }
  return miny;
}


// tensile stress
double T_(double ds, double hs) {
  double h_ = ds*(1-hs); // distance between crevasse water level and top of slab
  return (2*rho_ice*g/pi*(ds + h_*rho_H2O/rho_ice*(pi/2 - asin(h_/ds))
    - rho_H2O/rho_ice*sqrt(ds*ds - h_*h_)));
}


// main function
int main() {
  // user input
  double H, hw, hs, ds;
  std::cout << "\n  Height of ice slab (meters) = ";
  std::cin >> H;
  std::cout << "  Water level at terminus (percent) = ";
  std::cin >> hw;
  std::cout << "  Water level in crevasse (percent) = ";
  std::cin >> hs;
  // compute crevasse depth
  hw *= (0.01*H);
  hs *= 0.01;
  ds = Bisection(H, hw, hs);
  std::cout << "\n  Absolute crevasse depth:   " << ds << " m" << std::endl;
  std::cout << "  Normalized crevasse depth: " << ds/H*100 << " %";
  std::cout << std::endl << std::endl;

  return 0;
}
